/* Ce programme est strictement identique au démonstrateur P05, mais pour gérer le bruiteur
   on n'utilise plus la facilité de l'instruction Tone. Le bénéfice est :
   • 1312 octets de moins consommés dans la mémoire de programme.
   • 23 octets de moins consommés dans la mémoire dynamique.

   Naturellement les branchements sont strictement identiques à ceux de P05 et l'utilisation
   est totalement identique.
   
Compilateur v1.7.9 :                  PGM               RAM dynamique
Avec Tone initial >>>>          11902 Octets (38%)      404 Octets (19%)
Sans Tone mais PWM >>>          10590 Octets (34%)      381 Octets (18%)*/ 

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "Rev : 04/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.

//------------------------- Variables du programme --------------------------
boolean Allumer, BP_actif, Clic_long;
byte Num_BP, Num_LED;
int CAN_sur_A0, CAN_sur_A3, Mem_A6, Mem_A7, SAV_A6, SAV_A7;
float Tension_lue;

unsigned long Ancien_Temps_Ecoule_depuis_RESET = 0;

void setup()  {
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); EteindreLED_Triple();
  //------------ Géération permanente du signal de test ------------
  analogWrite(Signal_Etalon,128); // Fréquence 490Hz et rapport cyclique 50%.
  //----------------- Initialisation des variables -----------------
  Allumer = false; BP_actif = false; Mem_A6 = 0; Mem_A7 = 0; Num_LED = 0;}

void loop() {
  Traiter_une_fois_par_seconde();
  CAN_sur_A0 = analogRead(A0); // Environ 100µS.
  CAN_sur_A3 = analogRead(A3); // Environ 100µS.
  CAN_sur_A3 = map(CAN_sur_A3, 0, 1023, 0, 5000);
  Tension_lue = float(CAN_sur_A3) / 1000;
  PAGE1();
  //-------------------------------------------------------------
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (BP_actif) Traiter_activation_du_BP();}

//------------------------ Procédures de servitude --------------------------

void EteindreLED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void BIP () {
  for(int N = 0; N < 600; N++)
     {digitalWrite(BUZZER, HIGH); delayMicroseconds(125);  digitalWrite(BUZZER, LOW); delayMicroseconds(125);}}

void PAGE1() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //================ Définition des éléments à afficher ================
  u8g.setPrintPos(10, 11); u8g.print(Version);
  u8g.setPrintPos(10, 23); u8g.print("CAN sur A0 = "); u8g.print(CAN_sur_A0);
  u8g.setPrintPos(10, 35); u8g.print("U sur  E = "); u8g.print(Tension_lue); ('V');
  u8g.setPrintPos(10, 47); u8g.print("BP = ");
  switch (Num_BP) {
    case 0 : {u8g.print("xxx"); break;}
    case 1 : {u8g.print("Bas"); break;}
    case 2 : {u8g.print("Haut"); break;}
    case 3 : {u8g.print("Gauche"); break;}
    case 4 : u8g.print("Droite");}
  if (Num_BP > 0)  {u8g.print(" : ");
      if (Clic_long) u8g.print("Long."); else u8g.print("Court.");}
  u8g.setPrintPos(10, 59); u8g.print("A6 = "); u8g.print(Mem_A6);// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  u8g.print("  A7 = "); u8g.print(Mem_A7);} //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  //u8g.setPrintPos(10, 59); u8g.print("A6 = "); u8g.print(analogRead(BP_A6));
  //u8g.print("  A7 = "); u8g.print(analogRead(BP_A7));}
 
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void Traiter_une_fois_par_seconde() {// Le délai est de une seconde.
  //==================== Chronométrage pour une seconde ===================
  if (millis() - Ancien_Temps_Ecoule_depuis_RESET > 1000)  {
     Ancien_Temps_Ecoule_depuis_RESET = millis(); // réaeme pour une seconde.
     //----------------- Actions à traiter pour chaque seconde écoulée. -----------------
     Num_LED++; if (Num_LED == 10) Num_LED = 0;
     switch(Num_LED) {
        case 1 : {digitalWrite(Trp_R,HIGH);break;}
        case 3 : {digitalWrite(Trp_V,HIGH);break;}
        case 5 : {digitalWrite(Trp_B,HIGH);break;}
        case 7 : {digitalWrite(Rouge,HIGH);break;}
        case 9 : digitalWrite(Verte,HIGH);}
     Allumer = !Allumer;
     if (!Allumer) {EteindreLED_Triple(); digitalWrite(Rouge,LOW); digitalWrite(Verte,LOW);}}}

void Traiter_activation_du_BP() {BP_actif = false;}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  analogWrite(BUZZER,128); // Tonalité grave durant le clic du B.P.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW); analogWrite(BUZZER,0);}

void Tester_le_BP() {
  BP_actif = false;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Num_BP = 4; BP_actif = true; Mem_A6 = SAV_A6;} // BP Droit.
                              if (SAV_A6 > 707) {Mem_A6 = SAV_A6;
                                  Num_BP = 2; BP_actif = true; BIP();} // BP Haut.
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Mem_A7 = SAV_A7; Num_BP = 1; BP_actif = true;} // BP Bas.
                              if (SAV_A7 > 708) {
                                  Mem_A7 = SAV_A7; Num_BP = 3; BP_actif = true;} // BP Gauche.
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}
  
