/* Ce programme est issu de P06 et permet de valider la procédure Lire_INDEX qui servira à
   traiter tous les menus. par ailleurs à partir d'ici c'est un bruiteur ACTIF qui est traité.
   La routine Lire_INDEX optimisée ne consomme que 136 octets de programme.
   
   Naturellement les branchements sont strictement identiques à ceux de P06 et l'utilisation
   est analogue à celle de P06 sauf que sur la ligne du bas c'est l'INDEX qui a été
   détecté qui est indiqué.
   
Compilateur v1.7.9 :                  PGM               RAM dynamique
Sans la routine Lire_INDEX :    10496 Octets (34%)      376 Octets (18%)
Avec la routine Lire_INDEX :    10632 Octets (34%)      376 Octets (18%)*/ 

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "Rev : 04/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.

//------------------------- Variables du programme --------------------------
boolean Allumer, BP_actif, Clic_long;
byte INDEX, Num_BP, Num_LED;
int CAN_sur_A0, CAN_sur_A3, SAV_A6, SAV_A7;
float Tension_lue;

unsigned long Ancien_Temps_Ecoule_depuis_RESET = 0;

void setup()  {
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); EteindreLED_Triple();
  //------------ Géération permanente du signal de test ------------
  analogWrite(Signal_Etalon,128); // Fréquence 490Hz et rapport cyclique 50%.
  //----------------- Initialisation des variables -----------------
  Allumer = false; BP_actif = false; INDEX = 0; Num_LED = 0;}

void loop() {
  Traiter_une_fois_par_seconde();
  CAN_sur_A0 = analogRead(A0); // Environ 100µS.
  CAN_sur_A3 = analogRead(A3); // Environ 100µS.
  CAN_sur_A3 = map(CAN_sur_A3, 0, 1023, 0, 5000);
  Tension_lue = float(CAN_sur_A3) / 1000;
  PAGE1();
  //-------------------------------------------------------------
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (BP_actif) Traiter_activation_du_BP();}

//------------------------ Procédures de servitude --------------------------

void EteindreLED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void BIP (byte Duree) {
  digitalWrite(BUZZER, HIGH); delay(Duree);digitalWrite(BUZZER, LOW);}

void PAGE1() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //================ Définition des éléments à afficher ================
  u8g.setPrintPos(10, 11); u8g.print(Version);
  u8g.setPrintPos(10, 23); u8g.print("CAN sur A0 = "); u8g.print(CAN_sur_A0);
  u8g.setPrintPos(10, 35); u8g.print("U sur  E = "); u8g.print(Tension_lue); ('V');
  u8g.setPrintPos(10, 47); u8g.print("BP = ");
  switch (Num_BP) {
    case 0 : {u8g.print("xxx"); break;}
    case 1 : {u8g.print("Bas"); break;}
    case 2 : {u8g.print("Haut"); break;}
    case 3 : {u8g.print("Gauche"); break;}
    case 4 : u8g.print("Droite");}
  if (Num_BP > 0)  {u8g.print(" : ");
      if (Clic_long) u8g.print("Long."); else u8g.print("Court.");}
  u8g.setPrintPos(10, 59); u8g.print("Position = "); u8g.print(INDEX);}
 
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void Traiter_une_fois_par_seconde() {// Le délai est de une seconde.
  //==================== Chronométrage pour une seconde ===================
  if (millis() - Ancien_Temps_Ecoule_depuis_RESET > 1000)  {
     Ancien_Temps_Ecoule_depuis_RESET = millis(); // réaeme pour une seconde.
     //----------------- Actions à traiter pour chaque seconde écoulée. -----------------
     Num_LED++; if (Num_LED == 10) Num_LED = 0;
     switch(Num_LED) {
        case 1 : {digitalWrite(Trp_R,HIGH);break;}
        case 3 : {digitalWrite(Trp_V,HIGH);break;}
        case 5 : {digitalWrite(Trp_B,HIGH);break;}
        case 7 : {digitalWrite(Rouge,HIGH);break;}
        case 9 : digitalWrite(Verte,HIGH);}
     Allumer = !Allumer;
     if (!Allumer) {EteindreLED_Triple(); digitalWrite(Rouge,LOW); digitalWrite(Verte,LOW);}}}

void Lire_INDEX() {
  long int Total = 0;
 // for (byte N=0; N<100; N++) {Total = Total + analogRead(A0);}
 // CAN_sur_A0 = int(Total/100);
  CAN_sur_A0 = analogRead(A0);
  if (CAN_sur_A0 < 96) INDEX = 1;
  if ((CAN_sur_A0 >= 96) && (CAN_sur_A0 < 309)) INDEX = 2;
  if ((CAN_sur_A0 >= 309) && (CAN_sur_A0 < 536)) INDEX = 3;
  if ((CAN_sur_A0 >= 536) && (CAN_sur_A0 < 748)) INDEX = 4;
  if ((CAN_sur_A0 >= 748) && (CAN_sur_A0 < 937)) INDEX = 5;
  if (CAN_sur_A0 > 936) INDEX = 6;}

void Traiter_activation_du_BP() {BP_actif = false; Lire_INDEX();}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  BIP(10); // Bip court.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW);}

void Tester_le_BP() {
  BP_actif = false;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Num_BP = 4; BP_actif = true;} // BP Droit.
                              if (SAV_A6 > 707) {
                                  Num_BP = 2; BP_actif = true; BIP(150);} // BP Haut.
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Num_BP = 1; BP_actif = true;} // BP Bas.
                              if (SAV_A7 > 708) {
                                  Num_BP = 3; BP_actif = true;} // BP Gauche.
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}
  
