/* Ce programme est prévu pour observer le gain de place que l'on réalise quand on loge
   les textes affichés par le logiciel en EEPROM.
   Naturellement les branchements sont strictement identiques à ceux des démonstrateurs
   qui précèdent.
   
Compilateur v1.7.9 :                  PGM               RAM dynamique
Textes dans le programme :      8534 Octets (27%)      948 Octets (46%)
Textes en EEPROM :              8392 Octets (27%)      764 Octets (37%)*/ 

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "08/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.
#define HAUT 2
#define OUI 2
#define BAS 1
#define NON 1
#define GAUCHE 3
#define DROIT 4

//------------------------- Variables du programme --------------------------
boolean Allumer, BP_actif, Clic_long;
byte INDEX, Touche, Num_LED;
int SAV_A6, SAV_A7;

byte TRACE[512];

void setup()  {Initialiser_la_TRACE();
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); EteindreLED_Triple();
  //------------ Géération permanente du signal de test ------------
  analogWrite(Signal_Etalon,128); // Fréquence 490Hz et rapport cyclique 50%.
  //----------------- Initialisation des variables -----------------
  Allumer = false; BP_actif = false;}

void loop() {
  PAGE1();
  //-------------------------------------------------------------
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (BP_actif) Traiter_activation_du_BP();}

//==================== Routines pour la gestion de l'EEPROM ====================
byte Lire_Octet_EEPROM(int PTR) {
  return eeprom_read_byte((unsigned char *) PTR);}
  
byte Ecrire_Octet_EEPROM(int PTR, byte OCTET) {
  eeprom_write_byte((unsigned char*) PTR,OCTET);}

void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {
    for (byte I=0; I<Longueur; I++) {u8g.print(char(Lire_Octet_EEPROM(PTR))); PTR++;};}
//==============================================================================

//======================== Procédures de servitude =============================
void Initialiser_la_TRACE() {
  int Valeur = 0;
  for (int Echantillon = 0; Echantillon < 512; Echantillon++)
    {TRACE[Echantillon] = Valeur; Valeur = Valeur + 4; if (Valeur > 254) Valeur = 0;}} 
       
void EteindreLED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void BIP (byte Duree) {
  digitalWrite(BUZZER, HIGH); delay(Duree);digitalWrite(BUZZER, LOW);}

void Attendre_un_BP() {Touche = 0; while (Touche == 0) Tester_le_BP();}

//=================== Routines pour les textes dans l'EEPROM ===================
/*
void PAGE1() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(0, 13); Aff_TEXTE_EEPROM(0, 6); u8g.print(Version); //"Rev : "
  u8g.setPrintPos(0, 25); Aff_TEXTE_EEPROM(6, 21); // "Trace actuelle : Haut"
  u8g.setPrintPos(0, 37); Aff_TEXTE_EEPROM(27, 21); // "Trace en EEPROM : Bas"
  u8g.setPrintPos(0, 49); Aff_TEXTE_EEPROM(48, 19); // "RAZ EEPROM : Gauche"
  u8g.setPrintPos(0, 61); Aff_TEXTE_EEPROM(127, 16);} //"Stocker : Droite"
  //--------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void Affiche_trace_Actuelle() {int octet;
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(10, 12); Aff_TEXTE_EEPROM(67, 20); // "Un B.P. pour sortir."
  for (byte X  =0; X < 128; X++) {// Ci-dessous : Diviser la valeur par 4.
      u8g.drawPixel(X,(255 - TRACE[X]) >> 2);}}  while(u8g.nextPage());
  //--------------------------------------------------------------------
  Attendre_un_BP();}

void Affiche_trace_EEPROM() {
  int PTR = 512;
  for (int Echantillon = 0; Echantillon < 512; Echantillon++)
    {TRACE[Echantillon] = Lire_Octet_EEPROM(PTR); PTR++;} 
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(28, 12); Aff_TEXTE_EEPROM(87, 13); // "Trace longue."
  for (byte X  =0; X < 128; X++) {// Ci-dessous : Diviser la valeur par 4.
      u8g.drawPixel(X,(255 - TRACE[X]) >> 2);}}  while(u8g.nextPage());
  //--------------------------------------------------------------------
  Attendre_un_BP();}

void Lignes_du_bas() {
  u8g.setPrintPos(20, 30); Aff_TEXTE_EEPROM(87, 13); // "Trace longue."
  u8g.setPrintPos(14, 48); Aff_TEXTE_EEPROM(100, 14); // "OUI : BP Haut."
  u8g.setPrintPos(14, 63); Aff_TEXTE_EEPROM(114, 13);} // "NON : BP Bas."

void Enregistrer_la_TRACE() {int PTR;
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(10, 13); Aff_TEXTE_EEPROM(127, 18); //"Stocker la TRACE ?"
  Lignes_du_bas();}
  //--------------------------------------------------------------------
  while(u8g.nextPage());
  PTR = 512; Attendre_un_BP();
  if (Touche == OUI) {digitalWrite(Trp_R,HIGH);
     for (int Echantillon = 0; Echantillon < 512; Echantillon++)
         {Ecrire_Octet_EEPROM(PTR, TRACE[Echantillon]); PTR++;}
     digitalWrite(Trp_R,LOW);}}

void Efface_trace_EEPROM() {int PTR = 512;
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(20, 18); Aff_TEXTE_EEPROM(48, 12); // "RAZ EEPROM :"
  Lignes_du_bas();}
  //--------------------------------------------------------------------
  while(u8g.nextPage());
  PTR = 512; Attendre_un_BP();
  if (Touche == HAUT) {digitalWrite(Trp_R,HIGH);
     for (int Echantillon = 0; Echantillon < 512; Echantillon++)
         {Ecrire_Octet_EEPROM(PTR, 0); PTR++;}
     digitalWrite(Trp_R,LOW);}}
*/
//================= Routines pour les textes dans le programme =================
void PAGE1() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(0, 13); u8g.print("Rev : ");  u8g.print(Version);
  u8g.setPrintPos(0, 25); u8g.print("Trace actuelle : Haut");
  u8g.setPrintPos(0, 37); u8g.print("Trace en EEPROM : Bas");
  u8g.setPrintPos(0, 49); u8g.print("RAZ EEPROM : Gauche");
  u8g.setPrintPos(0, 61); u8g.print("Stocker : Droite");}
  //--------------------------------------------------------------------
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void Affiche_trace_Actuelle() {int octet;
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(10, 12); u8g.print("Un B.P. pour sortir.");
  for (byte X  =0; X < 128; X++) {// Ci-dessous : Diviser la valeur par 4.
      u8g.drawPixel(X,(255 - TRACE[X]) >> 2);}}  while(u8g.nextPage());
  //--------------------------------------------------------------------
  Attendre_un_BP();}
  
void Affiche_trace_EEPROM() {
  int PTR = 512;
  for (int Echantillon = 0; Echantillon < 512; Echantillon++)
    {TRACE[Echantillon] = Lire_Octet_EEPROM(PTR); PTR++;} 
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(28, 12); u8g.print("Trace longue.");
  for (byte X  =0; X < 128; X++) {// Ci-dessous : Diviser la valeur par 4.
      u8g.drawPixel(X,(255 - TRACE[X]) >> 2);}}  while(u8g.nextPage());
  //--------------------------------------------------------------------
  Attendre_un_BP();}

void Lignes_du_bas() {
  u8g.setPrintPos(20, 33); u8g.print("Trace longue.");
  u8g.setPrintPos(14, 48); u8g.print("OUI : BP Haut.");
  u8g.setPrintPos(14, 63); u8g.print("NON : BP Bas.");}

void Enregistrer_la_TRACE() {int PTR;
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(10, 13); u8g.print("Stocker la TRACE ?");
  Lignes_du_bas();}
  //--------------------------------------------------------------------
  while(u8g.nextPage());
  PTR = 512; Attendre_un_BP();
  if (Touche == OUI) {digitalWrite(Trp_R,HIGH);
     for (int Echantillon = 0; Echantillon < 512; Echantillon++)
         {Ecrire_Octet_EEPROM(PTR, TRACE[Echantillon]); PTR++;}
     digitalWrite(Trp_R,LOW);}}

void Efface_trace_EEPROM() {int PTR = 512;
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(20, 18); u8g.print("RAZ EEPROM :");
  Lignes_du_bas();}
  //--------------------------------------------------------------------
  while(u8g.nextPage());
  PTR = 512; Attendre_un_BP();
  if (Touche == OUI) {digitalWrite(Trp_R,HIGH);
     for (int Echantillon = 0; Echantillon < 512; Echantillon++)
         {Ecrire_Octet_EEPROM(PTR, 0); PTR++;}
     digitalWrite(Trp_R,LOW);}}
//==============================================================================

void Traiter_activation_du_BP() {
  BP_actif = false;
  switch (Touche) {
    case BAS : {Affiche_trace_EEPROM(); break;}
    case HAUT : {Affiche_trace_Actuelle(); break;}
    case GAUCHE : {BIP(150); Efface_trace_EEPROM(); break;}
    case DROIT : {BIP(150); Enregistrer_la_TRACE();}}}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  BIP(10); // Bip court.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW);}

void Tester_le_BP() {
  BP_actif = false; Touche = 0;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Touche = DROIT; BP_actif = true;}
                              if (SAV_A6 > 707) {
                                  Touche = HAUT; BP_actif = true;}
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Touche = BAS; BP_actif = true;}
                              if (SAV_A7 > 708) {
                                  Touche = GAUCHE; BP_actif = true;}
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}
  
