  /* Ce programme est prévu pour développer le MENU de la BASE de TEMPS.
   Naturellement les branchements sont strictement identiques à ceux des démonstrateurs
   qui précèdent. Utilisation :
   • Tout BP fait passer du MENU de BASE à la fonction Initialisation de la BASE de TEMPS.
   • Dans la fonction Initialisation de la BASE de TEMPS :
        * DROITE fait revenir au MENU de BASE,
        * GAUCHE fait changer d'unités en permutation circulaire.
        * HAUT augmente la valeur en permutation circulaire dans les mantisses 1, 2 et 5.
        * BAS diminue la valeur en permutation circulaire dans les mantisses 1, 2 et 5.
   • Sur RESET la cadence la plus rapide est initialisée. (100µS correspondant à la durée
     de numérisation des échantillons. Des valeurs plus faibles seraient illusoires.)

   >>> Textes supposés en EEPROM cette fonction consomme environ 3300 Octets de programme soit
   >>> environ 10% de l'espace disponible. Environ 25 Octets dans la mémoire dynamique. 
   
Compilateur v1.7.9 :                  PGM               RAM dynamique
Textes dans le programme :      10472 Octets (34%)      315 Octets (15%) */

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "09/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.
#define HAUT 2
#define OUI 2
#define BAS 1
#define NON 1
#define GAUCHE 3
#define DROIT 4

//------------------------- Variables du programme --------------------------
boolean Clic_long;
byte Touche, Unites;
int BdT, SAV_A6, SAV_A7;

void setup()  {
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); EteindreLED_Triple();
  //------------ Géération permanente du signal de test ------------
  analogWrite(Signal_Etalon,128); // Fréquence 490Hz et rapport cyclique 50%.
  //----------------- Initialisation des variables -----------------
  BdT = 100; Unites = 1;}

void loop() {
  PAGE1();
  //-------------------------------------------------------------
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (Touche > 0) Gerer_la_Base_de_Temps();}

//======================== Procédures de servitude =============================
       
void EteindreLED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void BIP (byte Duree) {
  digitalWrite(BUZZER, HIGH); delay(Duree);digitalWrite(BUZZER, LOW);}

void Attendre_un_BP() {Touche = 0; while (Touche == 0) Tester_le_BP();}

void PAGE1() {
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(30, 35); u8g.print("MENU de BASE");}
  //--------------------------------------------------------------------
  while(u8g.nextPage());}

void Affiche_unites() {
    switch(Unites) {
    case 1 : {u8g.print(' '); u8g.print(char(181)); u8g.print('S'); break;}
    case 2 : {u8g.print(" mS"); break;}
    case 3 :  u8g.print(" S");}}

void PAGE2() { // Impossible d'avoir un résultat correcte de (BdT * 500) si Total est en
  // long, unsigned long. Toutes les combinaisons possibles de calcul avec des "entiers"
  // a engendré des résultats faux, y compris avec des calculs directs dans u8g.print().
  // Seule solution trouvée, passer par un float.
  float Total = BdT * 5.12;
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(23,13); u8g.print("BASE de TEMPS");
  u8g.setPrintPos(12,29); u8g.print("Dur");u8g.print(char(233)); u8g.print("e : ");
  u8g.print(BdT); Affiche_unites(); u8g.setPrintPos(12,45); u8g.print("Total : ");
  u8g.print(Total * 100,0); Affiche_unites(); u8g.setPrintPos(0, 58); u8g.print("Fen");
  u8g.print(char(234)); u8g.print("tre : "); u8g.print(unsigned(BdT * 128)); Affiche_unites();}
  //--------------------------------------------------------------------
  while(u8g.nextPage());}

void Diminue_BT() {
  if(Unites == 1) {switch (BdT) {
                      case 100 : {BdT = 500; break;}
                      case 200 : {BdT = 100; break;}
                      case 500 : BdT = 200;}}
  if(Unites == 2) {switch (BdT) {
                      case 1 : {BdT = 500; break;}
                      case 2 : {BdT = 1; break;}
                      case 5 : {BdT = 2; break;}
                      case 10 : {BdT = 5; break;}
                      case 20 : {BdT = 10; break;}
                      case 50 : {BdT = 20; break;}
                      case 100 : {BdT = 50; break;}
                      case 200 : {BdT = 100; break;}
                      case 500 : BdT = 200;}}
  if(Unites == 3) {switch (BdT) {
                      case 1 : {BdT = 5; break;}
                      case 2 : {BdT = 1; break;}
                      case 5 : BdT = 2;}}}

void Augmente_BT() {
  if(Unites == 1) {switch (BdT) {
                      case 100 : {BdT = 200; break;}
                      case 200 : {BdT = 500; break;}
                      case 500 : BdT = 100;}}
  if(Unites == 2) {switch (BdT) {
                      case 1 : {BdT = 2; break;}
                      case 2 : {BdT = 5; break;}
                      case 5 : {BdT = 10; break;}
                      case 10 : {BdT = 20; break;}
                      case 20 : {BdT = 50; break;}
                      case 50 : {BdT = 100; break;}
                      case 100 : {BdT = 200; break;}
                      case 200 : {BdT = 500; break;}
                      case 500 : BdT = 1;}}
  if(Unites == 3) {switch (BdT) {
                      case 1 : {BdT = 2; break;}
                      case 2 : {BdT = 5; break;}
                      case 5 : BdT = 1;}}}

void Gerer_la_Base_de_Temps() {// Unites : 1 en µS, 2 en mS et 3 en S.
  boolean FINI = false;
  while (!FINI) {
    Tester_le_BP();
    if (Touche > 0) {
        switch (Touche) {
          case DROIT : {FINI = true; break;}
          case BAS   : {Diminue_BT(); break;}
          case HAUT  : {Augmente_BT(); break;}
          case GAUCHE : {Unites++; if(Unites == 4) Unites = 1;
                         BdT = 1; if (Unites == 1) BdT = 100; break;}}}
    PAGE2();}}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  BIP(10); // Bip court.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW);}

void Tester_le_BP() {
  boolean BP_actif = false; Touche = 0;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Touche = DROIT; BP_actif = true;}
                              if (SAV_A6 > 707) {
                                  Touche = HAUT; BP_actif = true;}
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Touche = BAS; BP_actif = true;}
                              if (SAV_A7 > 708) {
                                  Touche = GAUCHE; BP_actif = true;}
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}
  
