  /* Ce programme est prévu pour développer le MENU de la sélection du mode de DÉCLENCHEMENT.
   Naturellement les branchements sont strictement identiques à ceux des démonstrateurs
   qui précèdent. Utilisation :
   • Tout BP fait passer du MENU de BASE à la fonction Initialisation du DÉCLENCHEMENT.
   • Dans la fonction Initialisation de la BASE de TEMPS :
        * DROITE fait revenir au MENU de BASE,
        * GAUCHE alterne entre Front montant et Front Descendant.
        * HAUT permet de choisir un seuil de tension pour le déclenchement. C'est la
          position du potentiomètre branché sur A0 qui définit la tension de déclenchement.
        * BAS > Si clic court : Alterne entre MANUEL et AUTOMatique.
                Si clic long : Impose le déclenchement LIBRE sans contrainte.
   • Sur RESET le mode est AUTOMatique et le déclenchement sans contrainte.

   >>> Textes supposés en EEPROM cette fonction consomme environ 3562 Octets de programme soit
   >>> environ 12% de l'espace disponible. 
   
Compilateur v1.7.9 :                  PGM               RAM dynamique
Textes dans le programme :      10432 Octets (33%)      375 Octets (18%) */

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "11/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.
#define HAUT 2
#define OUI 2
#define BAS 1
#define NON 1
#define GAUCHE 3
#define DROIT 4

//------------------------- Variables du programme --------------------------
boolean Clic_long, Libre, Manuel, Seuil;
byte Front, Tension, Touche;
int SAV_A6, SAV_A7;

void setup()  {
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); EteindreLED_Triple();
  //------------ Géération permanente du signal de test ------------
  analogWrite(Signal_Etalon,128); // Fréquence 490Hz et rapport cyclique 50%.
  //----------------- Initialisation des variables -----------------
  Manuel = false; Front = 0; Libre = true; Seuil = false;}

void loop() {
  PAGE1();
  //-------------------------------------------------------------
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (Touche > 0) Menu_Modes_de_DECLENCHEMENT();}

//======================== Procédures de servitude =============================
       
void EteindreLED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void BIP (byte Duree) {
  digitalWrite(BUZZER, HIGH); delay(Duree);digitalWrite(BUZZER, LOW);}

void Attendre_un_BP() {Touche = 0; while (Touche == 0) Tester_le_BP();}

void PAGE1() {
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(30, 35); u8g.print("MENU de BASE");}
  //--------------------------------------------------------------------
  while(u8g.nextPage());}

void PAGE2() {
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  u8g.setPrintPos(0, 13); u8g.print("Mode de D"); u8g.print(char(233)); u8g.print("clenchement");
  u8g.setPrintPos(6, 25); u8g.print("Mode : ");
  if (Manuel) u8g.print("MANUEL"); else u8g.print("AUTOM.");
  u8g.setPrintPos(0, 37); u8g.print("Seuil : ");
  if (Seuil) {u8g.print(float(Tension)/10,1); u8g.print('V');} else u8g.print("///");
  u8g.setPrintPos(0, 49); u8g.print("Front : ");
  switch (Front) {
    case 0 : {u8g.print("///"); break;}
    case 1 : {u8g.print("Montant");break;}
    case 2 : u8g.print("Descendant");}
  u8g.setPrintPos(0, 61); u8g.print("Libre : ");
  if (Libre) u8g.print("OUI"); else u8g.print("NON");}
  //--------------------------------------------------------------------
  while(u8g.nextPage());}


// Front = 0 : Libre   Front = 1 : Montant   Front = 2 : Descendant
void Menu_Modes_de_DECLENCHEMENT(){
  boolean FINI = false;
  while (!FINI) {
    Tester_le_BP();
    if (Touche > 0) {
        switch (Touche) {
          case DROIT : {FINI = true; break;}
          case BAS   : {if (Clic_long) {Libre = true; Front = 0; Seuil = false;} 
                        else Manuel = !Manuel; break;}
          case HAUT  : {Seuil = !Seuil; Libre = !Seuil; if (Libre) Front = 0; else Front = 1;
                        Tension = map(analogRead(A0), 0, 1020, 0 ,50); break;}
          case GAUCHE : {Libre = false; if (Front == 1) Front = 2; else Front = 1;}}}
    PAGE2();}}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  BIP(10); // Bip court.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW);}

void Tester_le_BP() {
  boolean BP_actif = false; Touche = 0;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Touche = DROIT; BP_actif = true;}
                              if (SAV_A6 > 707) {
                                  Touche = HAUT; BP_actif = true;}
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Touche = BAS; BP_actif = true;}
                              if (SAV_A7 > 708) {
                                  Touche = GAUCHE; BP_actif = true;}
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}
  
