  /* Ce programme est prévu pour développer les divers modes d'affichage. Ce programme ne
     traite aucune donnée, il se contente d'explorer quelque modes d'affichage.
   Naturellement les branchements sont strictement identiques à ceux des démonstrateurs
   qui précèdent. Utilisation :
   • DROIT avec clic court : Décale la fenêtre de visualisation vers la droite.
   • DROIT avec clic long : Change les unités.
   • GAUCHE avec clic court : Décale la fenêtre de visualisation vers la gauche.
   • GAUCHE avec clic long : Impose un Front ou le mode LIBRE.
   • HAUT valide ou non l'affichage de la configuration d'enregistrement.
   • BAS avec clic court : Affiche ou suspend la visualisation du graticule.
   • BAS avec clic long : Alterne entre représentation "filaire" ou "surface".
   • Sur RESET génère la TRACE d'une sinusoïde amortie avec décrément linéaire.
     Précise les valeurs des paramètres de synchronisation.
     Indique la sensibilité de saisie des échantillons et la "durée horizontale"
     par graduation. Affiche la grille et construit la courbe en mode "points".

   >>> Textes supposés en EEPROM cette fonction consomme environ 4380 Octets de programme soit
   >>> environ 14% de l'espace disponible. 
   
Compilateur v1.7.9 :                  PGM               RAM dynamique
Textes dans le programme :      11526 Octets (37%)      771 Octets (37%) */

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "11/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.
#define HAUT 2
#define OUI 2
#define BAS 1
#define NON 1
#define GAUCHE 3
#define DROIT 4

//------------------------- Variables du programme --------------------------
boolean Clic_long, Front_Montant, Grille, Seuil, Texte, Trace_pleine;
byte Amplitude, Front, Fenetre, Tension, Touche, Unites; // Tension : Valeur du seuil fois 10.
int BdT, SAV_A6, SAV_A7 ;
long Valeur_totale; // Pour calculer la valeur efficace.
byte TRACE [476]; // Graphe de 119 de large x 4 fenêtres.

void setup()  {
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); EteindreLED_Triple();
  //----------------- Initialisation des variables -----------------
  Amplitude = 5; BdT = 500; Fenetre = 0; Front = 0; Grille = true; Seuil = true; Tension = 25;
  Texte = true; Trace_pleine = false; Unites = 2;
  Genere_une_trace();}

void loop() {
  AFFICHE_TRACE();
  //-------------------------------------------------------------
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (Touche > 0) Traite_affichage();}

//======================== Procédures de servitude =============================
       
void EteindreLED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void BIP (byte Duree) {
  digitalWrite(BUZZER, HIGH); delay(Duree);digitalWrite(BUZZER, LOW);}

void Attendre_un_BP() {Touche = 0; while (Touche == 0) Tester_le_BP();}

void Genere_une_trace() {// Trace une sinusoïde amortie avec décrément linéaire.
  byte Amplitude;
  for (int Echantillon = 0; Echantillon < 476; Echantillon++) {
       Amplitude = 127 - (Echantillon >> 2); // >> 2 pour diviser la valeur par 4.
       TRACE[Echantillon] = 128 + (sin(radians(Echantillon * 6)) * Amplitude);}}
       // Ci-dessus la valeur 6 représente la fréquence du signal.

void Trace_la_courbe() {
  byte OCTET, Y; int Debut;
  Debut = Fenetre * 119; Valeur_totale = 0;
  for (byte X = 1; X < 120; X++) {
       OCTET = TRACE[(Debut + X)-1]; Y = map(OCTET,0,254,59,1);
       if(Trace_pleine) u8g.drawVLine(X, Y, 60-Y);
       else u8g.drawPixel(X, Y);
       Valeur_totale = Valeur_totale + OCTET;}}

void Trace_la_grille() {
  byte Debut = 1; if (Texte) Debut = 12;
  u8g.drawFrame(0,0,121,64); // Cadre extérieur.
  // Trace les cinq  graduations Horizontales.
  for (byte Y = 12; Y < 64; Y = Y+12) u8g.drawHLine(1, Y, 119);
  // Trace les neuf graduations Verticales.
  for (byte X = 12; X < 110; X = X+12) u8g.drawVLine(X, Debut, 60-Debut);
  // Trace les 3 graduations de délimitation des fenêtres.
  for (byte X = 30; X < 128; X = X+30) u8g.drawVLine(X, 61, 2);
  // Remplit l'un des quatre secteurs de positionnement des fenêtres.
  for(byte Ligne = 60; Ligne < 64; Ligne++) {
     u8g.drawHLine((Fenetre*30)+1 , Ligne, 29);}}

void AFFICHE_TRACE() {// Gère les options d'affichage.
  byte Y_Efficace = (map(byte(Valeur_totale / 119),0,254,59,1)) + 3 ;
  // Ci-dessus + 3 pour tenir compte de la hauteur du caractère '<'.
  u8g.firstPage(); do {
  //---------------- Définition des éléments à afficher ----------------
  if (Texte) {
      u8g.setPrintPos(6, 10); u8g.print(Amplitude); u8g.print("  "); u8g.print(BdT);
      if(Unites == 2) u8g.print('m'); u8g.print("S  ");
      if (Front == 0) {u8g.print(float(Tension)/10,1); u8g.print("V ");}
      else u8g.print("////  ");
      switch (Front) {
        case 0 : {u8g.print("//"); break;}
        case 1 : {u8g.print("FM");break;}
        case 2 : u8g.print("FD");}}
  if (Grille) Trace_la_grille();
  Trace_la_courbe();
  u8g.setPrintPos(122, Y_Efficace); u8g.print('<');} // Valeur efficace.
  //--------------------------------------------------------------------
  while(u8g.nextPage());}

void Traite_affichage() {
  switch (Touche) {
        case DROIT  : {if (Clic_long) {Unites++; if (Unites == 4) Unites = 2;}
                       else  {Fenetre++; if (Fenetre == 4) Fenetre = 0;} break;}
        case BAS    : {if (Clic_long) Trace_pleine = !Trace_pleine;
                       else Grille = !Grille; break;}
        case HAUT   : {Texte = !Texte; break;}
        case GAUCHE : {if (Clic_long) {Front++; if (Front == 3) Front = 0;}
                       else {Fenetre--; if (Fenetre == 255) Fenetre = 3;}}}}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  BIP(10); // Bip court.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW);}

void Tester_le_BP() {
  boolean BP_actif = false; Touche = 0;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Touche = DROIT; BP_actif = true;}
                              if (SAV_A6 > 707) {
                                  Touche = HAUT; BP_actif = true;}
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Touche = BAS; BP_actif = true;}
                              if (SAV_A7 > 708) {
                                  Touche = GAUCHE; BP_actif = true;}
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}
  
