  /* Ce programme est prévu pour ajouter à P13 les routines de saisie des
     echantillons et de leur stockage.
     
   Naturellement les branchements sont strictement identiques à ceux des démonstrateurs
   qui précèdent. Utilisation :
                  =============
   • Chaque fois que le logiciel attend un BP quelconque la LED triple bleue s'illumine.
   • Tout clic court fait passer du MENU de BASE à la fonction choisie.
   
   >>>> La fonction Initialisation de la BASE de TEMPS n'est plus émulée mais
   >>>> intégrée dans la fonction de saisie des échantillons.

   • Dans la fonction d'affichage graphique de l'enregistrement :
     ------------------------------------------------------------
     >>>> Le potentiomètre permet de décaler la trace en largeur jusqu'à une fenêtre.
     >>>> ATTENTION : Pour la fenêtre n°4 des valeurs quelconques sont entrées par la droite.
        * HAUT avec clic court valide ou non l'affichage de la ligne de texte.
        * HAUT avec clic long BIP d'erreur. (Non affecté.)        
        * BAS avec clic court : Affiche ou suspend la visualisation du graticule.
        * BAS avec clic long : Alterne entre représentation "filaire" ou "surface".
        * DROITE avec clic court Décale la fenêtre de visualisation vers la droite.      
        * DROITE avec clic long fait revenir au MENU de BASE.
        * GAUCHE avec clic court : Décale la fenêtre de visualisation vers la gauche.
        * GAUCHE avec clic long : BIP sonore d'erreur. (Non affecté.)
        * Si la trace descend en dessous de CAN = 1 allume la LED Verte.
        * Si la trace monte au dessus de CAN = 254 allume la LED Rouge.

   • Menu d'initialisation des contraintes de SYNCHRONISATION :
     ----------------------------------------------------------
     >>>> C'est avec un clic long sur la touche GAUCHE durant la fonction de
     >>>> numérisation que s'ouvre ce menu d'options.
     >>>> Clic court ou long ont même effet et la LED bleue s'illumine.
     >>>> Le potentiomètre ajuste la tension de seuil qui est mémorisée.
        * HAUT : Impose un Front Montant.        
        * BAS  : Impose un Front Descendant.
        * GAUCHE : Seuil OUI / NON.
        * DROITE : Sortie et retour au menu de numérisation.
 
   • Dans la fonction de numérisation :
     ----------------------------------
     >>>> En entrée de fonction on valide la fenêtre de gauche n°1.
     >>>> Durant la numérisation la LED triple Verte s'illumine.
     >>>> Durant la numérisation, si mode MANUEL la LED triple Rouge s'allume pour
     >>>> avertir l'opérateur que les trois touches H, B et G ne font que déclencher
     >>>> la numérisation et n'interviennent plus sur le Gain ou la BdT.
     >>>> Le mode Filaire/Surface défini lors de l'affichage de la TRACE est conservé.
        * HAUT avec clic court : Augmente la BdT en permutation circulaire 1, 2 et 5
        * HAUT avec clic long : Gain x1 / x2 / x4.        
        * BAS avec clic court : Diminue la BdT en permutation circulaire 1, 2 et 5
        * BAS avec clic long : Grille OUI/NON.
        * GAUCHE avec clic court ou long :
          Si automatique : Alterne entre LIBRE / SYNCHRONISÉ.
          Si MANUEL : Déclenche une numrisation.
        * DROITE avec clic court : MANUEL/AUTOMATIQUE.
        * DROITE avec clic long : Retour au MENU de BASE.
        
   • Sur RESET affiche le MENU de BASE. La TRACE du signal sera visualisé par défaut
     en mode filaire avec précision des valeurs de la sensibilité de saisie des échantillons,
     de la "durée horizontale" relative à une graduation sur le graticule et à la valeur
     efficace calculée pour la "largeur" de la fenêtre d'affichage. l'option de
     décalage latéral est activée par défaut.
   • Sur RESET avec un BP activé on invoque le Menu de la BASE de TEMPS. On modifie librement
     la base de temps pour faire afficher les durées totalisées par la mémorisation d'une
     trace complète. Mais en sortie la cadence de 100µS la plus rapide est initialisée soit
     une base de temps de 1.34mS.
   • Par défaut le Front Montant sera sélectionné.
   • Par défaut Seuil sera initialisé au false.
   
Compilateur v1.7.9 :                  PGM               RAM dynamique
Textes dans le programme :      14304 Octets (46%)      1010 Octets (49%) */

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "25/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.
#define HAUT 2
#define OUI 2
#define BAS 1
#define NON 1
#define GAUCHE 3
#define DROITE 4

//------------------------- Variables du programme --------------------------
boolean Automatique, Clic_long, Front_Montant, Grille, Libre, Mode_CAN, Seuil,
  Texte, Trace_pleine;
byte Fenetre, Gain, I_BdT, Touche, U_Synchronisation;
int BdT, Debut, Grandeur, SAV_A6, SAV_A7;
long Total, Valeur_totale; // Pour calculer la valeur efficace.
float Valeur; // Valeur de la tension mesurée.

byte TRACE [476]; // Graphe de 119 de large x 4 fenêtres.
  int Valeurs_BT[8] {1, 53, 301, 714, 1540, 4020, 8152, 16380};
  // 16380 est la plus grande valeur acceptable par l'instruction delayMicroseconds.

void setup()  {
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); EteindreLED_Triple();
  //----------- Génération permanente du signal de test ------------
  analogWrite(Signal_Etalon,128); // Fréquence 490Hz et rapport cyclique 50%.
  //------------- Option sur RESET si un BP est activé -------------
  Tester_le_BP(); if (Touche > 0) Menu_duree_BdT();
  //----------------- Initialisation des variables -----------------
  Fenetre = 0; Front_Montant = true; Gain = 2; Grille = true; BdT = 0;
  Mode_CAN = false; Seuil = false; Texte = true; Trace_pleine = false;}

void loop() {
  Aff_MENU_de_BASE();
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (Touche > 0) Gerer_le_MENU_de_BASE();}

//======================== Procédures de servitude =============================

// NOTE : Bien que u8g.print(' '); soit unilisée à plusieurs reprises, remplacer
//        chaque instruction  par un appel à une procédure n'est pas rentable.
     
void EteindreLED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void Bruiteur (byte Duree) {
  digitalWrite(BUZZER, HIGH); delay(Duree);digitalWrite(BUZZER, LOW);}

void BIP() {Bruiteur(150);}

void Attendre_un_BP() {Touche = 0;
  digitalWrite(Trp_B,HIGH); while (Touche == 0) Tester_le_BP(); digitalWrite(Trp_B,LOW);}

//======================== GESTION de la BASE de TEMPS ========================

void Affiche_la_duree() {
  if (I_BdT == 0) u8g.print("1.35mS");
  else {switch (I_BdT) {
          case 1 : {Grandeur = 2; break;}
          case 2 : {Grandeur = 5; break;}  
          case 3 : {Grandeur = 10; break;}
          case 4 : {Grandeur = 20; break;}
          case 5 : {Grandeur = 50; break;}
          case 6 : {Grandeur = 100; break;}
          case 7 : {Grandeur = 200;}} 
        u8g.print(Grandeur); u8g.print("mS");}} 

void Diminue_la_BT() {I_BdT--; if(I_BdT == 255) I_BdT = 7;}

void Augmente_la_BT() {I_BdT++; if(I_BdT == 8) I_BdT = 0;}
  
//===================== GESTION de l'affichage de la TRACE ====================

void Visualise_la_Trace() {
  byte OCTET, Y;
  for (byte X = 1; X < 120; X++) {
       OCTET = TRACE[(Debut + X)-1]; Y = map(OCTET,0,254,59,1);
       if(OCTET < 1) digitalWrite(Verte,HIGH); if(OCTET > 254) digitalWrite(Rouge,HIGH);
       if(Trace_pleine) u8g.drawVLine(X, Y, 60-Y);
       else u8g.drawPixel(X, Y);
       Valeur_totale = Valeur_totale + OCTET;}
  digitalWrite(Rouge,LOW); digitalWrite(Verte,LOW);}

void Trace_la_courbe() {
  Debut = 0; Valeur_totale = 0;
  Visualise_la_Trace();}

void Affiche_la_TRACE() {
  long Total = 0; Debut = Fenetre * 119; Valeur_totale = 0;
  for (byte Nb = 0; Nb < 255; Nb ++) Total = Total + (analogRead(A0) >> 3);
  Debut = Debut + (Total >> 8); // Décale la trace en latéral.
  Visualise_la_Trace();}

void Trace_la_grille() {
  byte Debut = 1; if (Texte || Mode_CAN) Debut = 12;
  u8g.drawFrame(0,0,121,64); // Cadre extérieur.
  // Trace les cinq  graduations Horizontales.
  for (byte Y = 12; Y < 64; Y = Y+12) u8g.drawHLine(1, Y, 119);
  // Trace les neuf graduations Verticales.
  for (byte X = 12; X < 110; X = X+12) u8g.drawVLine(X, Debut, 60-Debut);
  // Trace les 3 graduations de délimitation des fenêtres.
  for (byte X = 30; X < 128; X = X+30) u8g.drawVLine(X, 61, 2);
  // Remplit l'un des quatre secteurs de positionnement des fenêtres.
  for(byte Ligne = 60; Ligne < 64; Ligne++) {
     u8g.drawHLine((Fenetre*30)+1 , Ligne, 29);}}

void Affiche_le_Gain(){
  switch(Gain) {
      case 0 : {u8g.print("1.25"); break;}
      case 1 : {u8g.print("2.5"); break;}
      case 2 : u8g.print("5");} u8g.print("v ");}

void AFFICHE_TRACE() {// Gère les options d'affichage.
  byte Y_Efficace = (map(byte(Valeur_totale / 119),0,254,59,1)) + 3 ;
  byte Diviseur;
  // Ci-dessus + 3 pour tenir compte de la hauteur du caractère '<'.
  u8g.firstPage(); do {
      if (Texte) {
         u8g.setPrintPos(5,10); Affiche_le_Gain();
         Affiche_la_duree(); u8g.print(' '); 
         switch(Gain) {
            case 0 : {Diviseur = 40; break;}
            case 1 : {Diviseur = 20; break;}
            case 2 : Diviseur = 10;}
         u8g.print((float(map(Valeur_totale / 119,0,254,0,50))) / Diviseur,1);
         u8g.print("vEff");}
      if (Grille) Trace_la_grille(); Affiche_la_TRACE();
      u8g.setPrintPos(122, Y_Efficace); u8g.print('<');} // Valeur efficace.
  //--------------------------------------------------------------------
  while(u8g.nextPage());}

void Traite_affichage() {
  boolean FINI = false;
  digitalWrite(Trp_V,HIGH); 
  while (!FINI) {
     Tester_le_BP();
     switch (Touche) {
        case DROITE  : {if (Clic_long) FINI = true;
                       else {Fenetre++; if (Fenetre == 4) Fenetre = 0;} break;}
        case BAS    : {if (Clic_long) Trace_pleine = !Trace_pleine;
                       else Grille = !Grille; break;}
        case HAUT   : {if (Clic_long) BIP();
                       else Texte = !Texte; break;}
        case GAUCHE : {if (Clic_long) BIP();
                       else {Fenetre--; if (Fenetre == 255) Fenetre = 3;}}}
     AFFICHE_TRACE();}
   digitalWrite(Trp_V,LOW);}
 
//========================== Capture des échantilons ==========================

void Affiche_numeriser() {u8g.print("Num"); u8g.print(char(233)); u8g.print("riser");}    
      
void Effectue_une_numerisation() {
    digitalWrite(Trp_V,HIGH);
    int TBT = Valeurs_BT[I_BdT];
    for (int PTR = 0; PTR < 476; PTR++) {
         TRACE[PTR] = analogRead(A3) >> Gain; delayMicroseconds(TBT);}
    digitalWrite(Trp_V,LOW);}
  
//=================== GESTION des options de SYNCHRONISATION ==================

void Affiche_la_tension_de_Seuil() {
  u8g.print(float(map(U_Synchronisation, 1,255, 1, 50))/10,1); u8g.print("V ");}

void Affiche_la_syncronisation() {
  digitalWrite(Trp_B,HIGH);
  u8g.firstPage(); do {
      for (byte Y = 1; Y < 64; Y = Y+12) u8g.drawHLine(120,Y,8);
      u8g.drawFrame(110,0,9,64); // Gauge verticale.
      u8g.drawBox(111,62-(U_Synchronisation >>2),7,63); // Niveau du seuil.
      u8g.drawFrame(60,19,24,13); u8g.setPrintPos(66,29);
      if (Front_Montant) u8g.print("FM"); else u8g.print("FD");
      u8g.setPrintPos(10,12); u8g.print("SYNCHRONISATION.");
      u8g.setPrintPos(0,25); u8g.print("H : FM.");
      u8g.setPrintPos(0,37); u8g.print("B : FD.");
      u8g.setPrintPos(0,49); u8g.print("G : Seuil > ");
      if (Seuil) u8g.print("OUI"); else u8g.print("NON");
      u8g.setPrintPos(0,61); u8g.print("D : Sortie.  ");
      Affiche_la_tension_de_Seuil();}
  while(u8g.nextPage()); EteindreLED_Triple();}
  
void Gerer_la_synchronisation() {
  boolean FINI = false;
  while (!FINI) {
    U_Synchronisation = map(analogRead(A0), 0,1023, 1, 255);
    Affiche_la_syncronisation();
    Tester_le_BP();
    switch (Touche) {
        case HAUT : {Front_Montant = true; break;}
        case BAS : {Front_Montant = false; break;}
        case GAUCHE : {Seuil= !Seuil; break;}
        case DROITE  : FINI = true;}}}

//================= Attente pour déclencher un enregistrement ================
// Si synchronisation attendre que les contraintes soient satisfaites.

void Gere_le_Declenchement() {
    byte U_Actuelle;
  //----------------- Déclenchement sur Seuil -----------------
  if (Seuil) {
     if (Front_Montant) {
         //------- Attendre U en entrée minimale ------
         U_Actuelle = analogRead(A3) >> 2;
         while ((analogRead(A3) >> 2) > U_Actuelle)
               {U_Actuelle = analogRead(A3) >> 2;}
         while ((analogRead(A3) >> 2) <= U_Actuelle) {
               U_Actuelle = analogRead(A3) >> 2;} // Attendre la valeur minimale.
         //--------------------------------------------
         while ((analogRead(A3) >> 2) < U_Synchronisation) {}} // Attendre le Seuil.        
     else { //------- Attendre U en entrée minimale ------
         while ((analogRead(A3) >> 2) < U_Actuelle)
               {U_Actuelle = analogRead(A3) >> 2;}
         while ((analogRead(A3) >> 2) >= U_Actuelle) {
               U_Actuelle = analogRead(A3) >> 2;} // Attendre la valeur maximale.
         //--------------------------------------------
         while ((analogRead(A3) >> 2) > U_Synchronisation) {}}} // Attendre le Seuil.
  //----------------- Déclenchement sur Front -----------------
  else {if (Front_Montant) while ((analogRead(A3) >> 2) < 128); // Ne rien faire.
        else while ((analogRead(A3) >> 2) > 128);}} // Ne rien faire.

//=============================================================================

void Saisie_des_echantillons() {
  // Gain représente le nombre de décalages à droite donc pour
  // Gain = 2,1,0 le signale sera "multiplié" par 0 2 et 4.
  boolean FINI = false; boolean SAV_Grille = Grille; Mode_CAN = true; Fenetre = 0;
  Automatique = true; Grille = false; Libre = true; Gain = 2; // 5V par défaut.
  while (!FINI) {
     if (!Libre) Gere_le_Declenchement();
     Effectue_une_numerisation();
     Tester_le_BP();
     switch (Touche) {
        case HAUT : {if (Clic_long) {Gain--; if (Gain == 255) Gain = 2;}
                     else Augmente_la_BT(); break;}
        case BAS : {if (Clic_long)  Grille = !Grille; else Diminue_la_BT(); break;}
        case GAUCHE : {if (Clic_long) Gerer_la_synchronisation();
                       else {if (Automatique) Libre = !Libre;} break;}
        case DROITE  : if (Clic_long) FINI = true;
                      else {Automatique = !Automatique;
                            if (!Automatique) digitalWrite(Trp_R,HIGH);}}
     u8g.firstPage(); do {
        u8g.setPrintPos(7,10); Affiche_le_Gain(); Affiche_la_duree(); u8g.print(' ');
        if (Automatique) u8g.print('A'); else u8g.print('M'); u8g.print(' ');
        if (Libre) u8g.print('L');
        else {if (Seuil) u8g.print('N'); else u8g.print('B');
              u8g.print(' '); if (Front_Montant) u8g.print('M'); else u8g.print('D');}
        if (Grille) Trace_la_grille(); Trace_la_courbe();}
     while(u8g.nextPage());
     if (!Automatique) {Attendre_un_BP(); if (Touche == DROITE) {
                                             Automatique = true;  digitalWrite(Trp_R,LOW);}}}
   Mode_CAN = false; Grille = SAV_Grille;}

//=============================================================================

void Affiche_duree_totale() {
  if (Total >= 1000) {u8g.print(Total / 1000); u8g.print('S');}
  else {u8g.print(Total); u8g.print("mS");}}

void Menu_duree_BdT() {
  boolean FINI = false;
  while (!FINI) {
    u8g.firstPage(); do {
    u8g.setPrintPos(23,13); u8g.print("BASE de TEMPS");
    u8g.setPrintPos(12,29); u8g.print("Dur"); u8g.print(char(233)); u8g.print("e : ");
    Affiche_la_duree(); u8g.setPrintPos(12,44); u8g.print("Total : "); Total = Grandeur * 40;
    Affiche_duree_totale(); u8g.setPrintPos(0, 58); u8g.print("Fen"); u8g.print(char(234));
    u8g.print("tre : "); Total = Grandeur * 10; Affiche_duree_totale();}
    while(u8g.nextPage());
  Attendre_un_BP();
  switch (Touche) {
    case HAUT : {Augmente_la_BT(); break;}
    case BAS  : {Diminue_la_BT(); break;}
    case GAUCHE : {BIP(); break;}
    case DROITE  : FINI = true;}}}

void Aff_MENU_de_BASE() {
  digitalWrite(Trp_B,HIGH);
  u8g.firstPage(); do {
      u8g.setPrintPos(20,13); u8g.print("MENU de BASE.");
      u8g.setPrintPos(0,25); u8g.print("HAUT : "); Affiche_numeriser(); u8g.print('.');
      u8g.setPrintPos(0,37); u8g.print("BAS : Afficher TRACE.");
      u8g.setPrintPos(0,49); u8g.print("GAUCHE : Menu EEPROM.");
      u8g.setPrintPos(0,61); u8g.print("DROITE : Sortie USB.");}
  while(u8g.nextPage()); EteindreLED_Triple();}

void Gerer_le_MENU_de_BASE() {
  switch (Touche) {
          case DROITE : {BIP(); break;}
          case GAUCHE : {BIP(); break;}
          case HAUT  : {Saisie_des_echantillons(); break;}
          case BAS   : Traite_affichage();}}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Bruiteur(10); // Bip court.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW);}

void Tester_le_BP() {
  boolean BP_actif = false; Touche = 0;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Touche = DROITE; BP_actif = true;}
                              if (SAV_A6 > 707) {
                                  Touche = HAUT; BP_actif = true;}
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Touche = BAS; BP_actif = true;}
                              if (SAV_A7 > 708) {
                                  Touche = GAUCHE; BP_actif = true;}
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}


  
