
  /* Ce programme est prévu pour ajouter à P14 les routines de sauvegarde et de
     rechargement des échantillons en EEPROM.
     NOTE : Placer un texte en EEPROM est rentable à partir de deux caractères.
     
   Naturellement les branchements sont strictement identiques à ceux des démonstrateurs
   qui précèdent. Utilisation :
                  =============
   • Commencer par téléverser P00_Textes_en_EEPROM.ino.
   • Activer le Moniteur de l'IDE pour exécuter le programme. (Inscrire les textes en EEPRoM.)
   • Téléverser P15 ce démonstrateur. Le MENU de BASE doit s'afficher.

   • Dans la fonction d'affichage graphique de l'enregistrement :
     ------------------------------------------------------------
     Protocoles inchangés par rapport à celui de P14.
     Toutefois si le rechargement est en mode FENETRE individuelle, chaque changement de fenêtre
     visualisera les attributs correspondants et le décalage latéral avec le potentiomètre
     n'est plus actif. 
   • Menu d'initialisation des contraintes de SYNCHRONISATION :
     ----------------------------------------------------------
     Protocoles inchangés par rapport à celui de P14.
   • Dans la fonction de numérisation :
     ----------------------------------
     Protocoles inchangés par rapport à celui de P14.
     
   • MENU EEPROM :  (Appel avec le Clic GAUCHE dans le MENU de BASE.)
     -------------
     * Ignore clic long ou clic court.
     * La LED triple s'allume en vert pour montrer le changement de menu.
     * La ligne du bas précise la nature de la trace en EEPROM :
       (Longue TRACE.) ou (Pas de TRACE !) ou (Quatre TRACES.)
     * B.P. HAUT : Sous-menu d'EFFACEMENT. La LED triple passe en rouge.
       Accepter avec OUI efface la totalité de la trace et retour au MENU de BASE la
       LED tricolore s'éclairant à nouveau en bleu.
       Refuser avec BAS ou DROITE ramène au sous menu EEPROM et la LED tripe repasse en vert.
       Accepter avec HAUT efface l'EEPROM et retour au MENU de BASE LED tripe en bleu.
     * B.P. GAUCHE active le sous menu de RECHARGEMENT et la LED triple s'illumine en bleu cyan.
       Ignore clic long ou clic court.  
       Accepter avec OUI recharge la totalité des échantillons et ramène au MENU de BASE le
       LED triple retrouvant la couleur bleue. Les quatre status sont rechargés. Si
       l'enregistrement est de type Long Gain et I_BdT sont ceux de ce dernier. Si l'enregistrement
       est individuel, La trace n°1 est imposée avec ses attributs.
       Refuser avec BAS ou DROITE ramène directement
       dans le MENU de BASE la LED triple retrouvant sa couleur bleue.
     * B.P. DROIT active le sous menu de SAUVEGARDE et la LED triple s'illumine en rouge.
       Ignore clic long ou clic court.
       DROITE ramène directement dans le MENU de BASE la LED triple retrouvant sa couleur bleue.
       GAUCHE change le numéro de la FENETRE pour une sauvegarde individuelle.
       HAUT : Valide la SAUVEGARDE.
              Si 4 traces en EEPROM et trace Longue à sauvegarder ouvre un Menu de confirmation.
              Si trace Longue en EEPROM sauvegarde 476 échantillons et retour au MENU de BASE.
              Que ce soit en Trace Longue ou individuelle les attributs sont sauvegardés.
       
   • Sur RESET : Comportement inchangés par rapport à celui de P14 sauf pour le
                 mode Surface qui maintenant est par défaut jugé plus "visuel".
                 Mode Fenêtre initialisé à faux par défaut.
   
Compilateur v1.7.9 :                     PGM               RAM dynamique
Textes dans le programme :         14304 Octets (46%)      1010 Octets (49%)
Textes passés en EEPROM  :         14026 Octets (45%)      776 Octets (37%)
Ajout Menu EEPROM et Effacement :  14766 Octets (48%)      776 Octets (37%)
Ajout Rechargement / Sauvegarde :  16526 Octets (53%)      786 Octets (38%) */

#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//--------------------- Définition des broches d'E/S ------------------------
#define Jaune 2
#define Rouge 3
#define BUZZER 4
#define Verte 5
#define Trp_R 6
#define Trp_V 7
#define Trp_B 8
#define Signal_Etalon 9

#define LED_Arduino 13  // D13 pour surveiller la boucle de base.
#define A0 14  // A0 : Entrée potentiomètre.
#define A3 17  // A3 : Entrée voltmètre.
#define BP_A6 20        // Entrée analogique A6 pour tester Le BP Gauche et le BP Droit.
#define BP_A7 21        // Entrée analogique A7 pour tester Le BP Haut et le BP BAS.

//------------------------ Constantes du programme --------------------------
#define Version "29/11/2023" //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation du B.P.
#define HAUT 2
#define OUI 2
#define BAS 1
#define NON 1
#define GAUCHE 3
#define DROITE 4

//------------------------- Variables du programme --------------------------
boolean Automatique, Clic_long, Fenetre, Front_Montant, Grille, Libre, Mode_CAN,
  Mode_Fenetre, Seuil, Texte, Trace_pleine;
byte Gain, Gain_1, Gain_2, Gain_3, Gain_4, I_BdT, I_BdT_1, I_BdT_2, I_BdT_3, I_BdT_4,
     Num_Fenetre, Touche, U_Synchronisation;
int BdT, Debut, Grandeur, SAV_A6, SAV_A7;
long Total, Valeur_totale; // Pour calculer la valeur efficace.
float Valeur; // Valeur de la tension mesurée.

byte TRACE [476]; // Graphe de 119 de large x 4 fenêtres.
  int Valeurs_BT[8] {1, 53, 301, 714, 1540, 4020, 8152, 16380};
  // 16380 est la plus grande valeur acceptable par l'instruction delayMicroseconds.

void setup()  {
  u8g.setFont(u8g_font_6x10); // Polices de 4x6 pixels.
  pinMode(LED_Arduino,OUTPUT); pinMode(Signal_Etalon,OUTPUT);
  pinMode(Jaune,OUTPUT); digitalWrite(Jaune,LOW); pinMode(Rouge,OUTPUT); digitalWrite(Rouge,LOW);
  pinMode(BUZZER,OUTPUT); digitalWrite(BUZZER,LOW); pinMode(Verte,OUTPUT); digitalWrite(Verte,LOW);
  pinMode(Trp_R,OUTPUT); pinMode(Trp_V,OUTPUT); pinMode(Trp_B,OUTPUT); Eteindre_LED_Triple();
  //----------- Génération permanente du signal de test ------------
  analogWrite(Signal_Etalon,128); // Fréquence 490Hz et rapport cyclique 50%.
  //------------- Option sur RESET si un BP est activé -------------
  Tester_le_BP(); if (Touche > 0) Menu_duree_BdT();
  //----------------- Initialisation des variables -----------------
  Front_Montant = true; Fenetre = false; Gain = 2; Grille = true; BdT = 0; Mode_CAN = false;
  Mode_Fenetre = false; Num_Fenetre = 1; Seuil = false; Texte = true; Trace_pleine = true;}

void loop() {
  Aff_MENU_de_BASE();
  digitalWrite(LED_Arduino,HIGH); delay(1); digitalWrite(LED_Arduino,LOW);
  Tester_le_BP(); if (Touche > 0) Gerer_le_MENU_de_BASE();}

//==================== Routines pour la gestion de l'EEPROM ====================
byte Lire_Octet_EEPROM(int PTR) {
  return eeprom_read_byte((unsigned char *) PTR);}
  
byte Ecrire_Octet_EEPROM(int PTR, byte OCTET) {
  eeprom_write_byte((unsigned char*) PTR,OCTET);}

void Aff_TEXTE_EEPROM(int PTR, byte Longueur) {
    for (byte I=0; I<Longueur; I++) {u8g.print(char(Lire_Octet_EEPROM(PTR))); PTR++;};}

//======================== Procédures de servitude =============================

// NOTE : Bien que u8g.print(' '); soit unilisée à plusieurs reprises, remplacer
//        chaque instruction  par un appel à une procédure n'est pas rentable.
     
void Eteindre_LED_Triple() {
  digitalWrite(Trp_R,LOW); digitalWrite(Trp_V,LOW); digitalWrite(Trp_B,LOW);}

void Bruiteur (byte Duree) {
  digitalWrite(BUZZER, HIGH); delay(Duree);digitalWrite(BUZZER, LOW);}

void BIP() {Bruiteur(150);}

void Attendre_un_BP() {Touche = 0;
  digitalWrite(Trp_B,HIGH); while (Touche == 0) Tester_le_BP(); digitalWrite(Trp_B,LOW);}

void Aff_NON() {Aff_TEXTE_EEPROM(168,3);} // "NON"

void Aff_OUI() {Aff_TEXTE_EEPROM(165,3);} // "OUI"

void Aff_Pas_de_TRACE() {Aff_TEXTE_EEPROM(210,14);} // "Pas de TRACE !"

void Aff_HAUT_deux_points_espace() {Aff_TEXTE_EEPROM(13,7);} // "HAUT : "

void DROITE_deux_points_espace() {Aff_TEXTE_EEPROM(62,9);} // "DROITE : "

void DROITE_deux_points_espace_Sortie_point() {
     DROITE_deux_points_espace(); Aff_TEXTE_EEPROM(156,7);} // "DROITE : Sortie."

void Aff_HAUT_deux_points_espace_OUI_Point() {
     Aff_HAUT_deux_points_espace(); Aff_OUI(); Point();} // "HAUT : OUI."

void Aff_BAS_deux_points_espace() {Aff_TEXTE_EEPROM(20,6);} // "BAS : "

//======================== GESTION de la BASE de TEMPS ========================

void Affiche_la_duree() {
  if (I_BdT == 0) Aff_TEXTE_EEPROM(82,6); // "1.35mS"
  else {switch (I_BdT) {
          case 1 : {Grandeur = 2; break;}
          case 2 : {Grandeur = 5; break;}  
          case 3 : {Grandeur = 10; break;}
          case 4 : {Grandeur = 20; break;}
          case 5 : {Grandeur = 50; break;}
          case 6 : {Grandeur = 100; break;}
          case 7 : {Grandeur = 200;}} 
          u8g.print(Grandeur); Aff_TEXTE_EEPROM(86,2);}} // "mS"

void Diminue_la_BT() {I_BdT--; if(I_BdT == 255) I_BdT = 7;}

void Augmente_la_BT() {I_BdT++; if(I_BdT == 8) I_BdT = 0;}
  
//===================== GESTION de l'affichage de la TRACE ====================

void Visualise_la_Trace() {
  byte OCTET, Y;
  for (byte X = 1; X < 120; X++) {
       OCTET = TRACE[(Debut + X)-1];
       if(OCTET < 1) digitalWrite(Verte,HIGH); if(OCTET > 254) digitalWrite(Rouge,HIGH);
       Y = map(OCTET,0,254,59,1);
       if(Trace_pleine) u8g.drawVLine(X, Y, 60-Y);
       else u8g.drawPixel(X, Y);
       Valeur_totale = Valeur_totale + OCTET;}
  digitalWrite(Rouge,LOW); digitalWrite(Verte,LOW);}

void Trace_la_courbe() {
  Debut = 0; Valeur_totale = 0;
  Visualise_la_Trace();}

void Calcule_le_Debut() {
  long Total = 0;
  Debut = (Num_Fenetre - 1) * 119; Valeur_totale = 0;
  // Ci-dessous fait la moyenne sur 255 mesures du potentiomètre.
  for (byte Nb = 0; Nb < 255; Nb ++) Total = Total + (analogRead(A0) >> 3);
  Debut = Debut + (Total >> 8);} // Décale la trace en latéral.

void Affiche_la_TRACE() {Calcule_le_Debut(); Visualise_la_Trace();}

void Trace_la_grille() {
  byte Debut = 1; if (Texte || Mode_CAN) Debut = 12;
  u8g.drawFrame(0,0,121,64); // Cadre extérieur.
  // Trace les cinq  graduations Horizontales.
  for (byte Y = 12; Y < 64; Y = Y+12) u8g.drawHLine(1, Y, 119);
  // Trace les neuf graduations Verticales.
  for (byte X = 12; X < 110; X = X+12) u8g.drawVLine(X, Debut, 60-Debut);
  // Trace les 3 graduations de délimitation des fenêtres.
  for (byte X = 30; X < 128; X = X+30) u8g.drawVLine(X, 61, 2);
  // Remplit l'un des quatre secteurs de positionnement des fenêtres.
  for(byte Ligne = 60; Ligne < 63; Ligne++) {
     u8g.drawHLine(((Num_Fenetre - 1)*30)+1 , Ligne, 29);}}

void Affiche_le_Gain(){
  switch(Gain) {
      case 0 : {Aff_TEXTE_EEPROM(88,4); break;} // "1.25" break;}
      case 1 : {Aff_TEXTE_EEPROM(92,3); break;} // "2.5")
      case 2 : u8g.print('5');} Aff_TEXTE_EEPROM(95,2);} // "v ")

void AFFICHE_TRACE() {// Gère les options d'affichage.
  byte Y_Efficace = (map(byte(Valeur_totale / 119),0,254,59,1)) + 3 ;
  byte Diviseur;
  // Ci-dessus + 3 pour tenir compte de la hauteur du caractère '<'.
  u8g.firstPage(); do {
      if (Texte) {
         if (Mode_Fenetre) {// Affiche les attributs individuels.
            switch(Num_Fenetre) {
               case 1 : {Gain = Gain_1; I_BdT = I_BdT_1; break;}
               case 2 : {Gain = Gain_2; I_BdT = I_BdT_2; break;}
               case 3 : {Gain = Gain_3; I_BdT = I_BdT_3; break;}
               case 4 : {Gain = Gain_4; I_BdT = I_BdT_4;}}}
         u8g.setPrintPos(5,10); Affiche_le_Gain(); Affiche_la_duree(); u8g.print(' ');
         //-------- Affiche la valeur numérique de Vefficace --------
         switch(Gain) {
            case 0 : {Diviseur = 40; break;}
            case 1 : {Diviseur = 20; break;}
            case 2 : Diviseur = 10;}
            u8g.print((float(map(Valeur_totale / 119,0,254,0,50))) / Diviseur,1);
            Aff_TEXTE_EEPROM(97,3);} // "Vef"
         //----------------------------------------------------------
      if (Mode_Fenetre) {u8g.setPrintPos(122,10); u8g.print(Num_Fenetre);
                         u8g.drawFrame(120,0,9,13);}
      if (Grille) Trace_la_grille(); Affiche_la_TRACE();
      u8g.setPrintPos(122, Y_Efficace); u8g.print('<');} // Valeur efficace.
  //--------------------------------------------------------------------
  while(u8g.nextPage());}

void Traite_affichage() {
  boolean FINI = false;
  digitalWrite(Trp_V,HIGH); 
  while (!FINI) {
     Tester_le_BP();
     switch (Touche) {
        case DROITE  : {if (Clic_long) FINI = true;
                       else {Num_Fenetre++; if (Num_Fenetre == 5) Num_Fenetre = 1;} break;}
        case BAS    : {if (Clic_long) Trace_pleine = !Trace_pleine;
                       else Grille = !Grille; break;}
        case HAUT   : {if (Clic_long) BIP();
                       else Texte = !Texte; break;}
        case GAUCHE : {if (Clic_long) BIP();
                       else {Num_Fenetre--; if (Num_Fenetre == 0) Num_Fenetre = 4;}}}
     AFFICHE_TRACE();}
   digitalWrite(Trp_V,LOW);}
 
//========================== Capture des échantilons ==========================

void Affiche_numeriser() {Aff_TEXTE_EEPROM(101,9);} // "Numériser"    
      
void Effectue_une_numerisation() {
    digitalWrite(Trp_V,HIGH);
    int TBT = Valeurs_BT[I_BdT];
    for (int PTR = 0; PTR < 476; PTR++) {
         TRACE[PTR] = analogRead(A3) >> Gain; delayMicroseconds(TBT);}
    digitalWrite(Trp_V,LOW);}
  
//=================== GESTION des options de SYNCHRONISATION ==================

void Affiche_la_tension_de_Seuil() {
  u8g.print(float(map(U_Synchronisation, 1,255, 1, 50))/10,1); Aff_TEXTE_EEPROM(95,2);} // "V ")

void Affiche_la_syncronisation() {
  digitalWrite(Trp_B,HIGH);
  u8g.firstPage(); do {
      for (byte Y = 1; Y < 64; Y = Y+12) u8g.drawHLine(120,Y,8);
      u8g.drawFrame(110,0,9,64); // Gauge verticale.
      u8g.drawBox(111,62-(U_Synchronisation >>2),7,63); // Niveau du seuil.
      u8g.drawFrame(60,19,24,13); u8g.setPrintPos(66,29);
      if (Front_Montant) Aff_TEXTE_EEPROM(130,2); // "FM"
      else Aff_TEXTE_EEPROM(137,2); // "FD")
      u8g.setPrintPos(10,12); Aff_TEXTE_EEPROM(110,16); // "SYNCHRONISATION.")
      u8g.setPrintPos(0,25); Aff_TEXTE_EEPROM(126,7); // "H : FM."
      u8g.setPrintPos(0,37); Aff_TEXTE_EEPROM(133,7); // "B : FD."
      u8g.setPrintPos(0,49); Aff_TEXTE_EEPROM(140,12); // "G : Seuil > "
      if (Seuil) Aff_OUI(); else Aff_NON();
      u8g.setPrintPos(0,61); Aff_TEXTE_EEPROM(152,13); // "D : Sortie.  "
      Affiche_la_tension_de_Seuil();}
  while(u8g.nextPage()); Eteindre_LED_Triple();}
  
void Gerer_la_synchronisation() {
  boolean FINI = false;
  while (!FINI) {
    U_Synchronisation = map(analogRead(A0), 0,1023, 1, 255);
    Affiche_la_syncronisation();
    Tester_le_BP();
    switch (Touche) {
        case HAUT : {Front_Montant = true; break;}
        case BAS : {Front_Montant = false; break;}
        case GAUCHE : {Seuil= !Seuil; break;}
        case DROITE  : FINI = true;}}}

//================= Attente pour déclencher un enregistrement ================
// Si synchronisation attendre que les contraintes soient satisfaites.

void Gere_le_Declenchement() {
    byte U_Actuelle;
  //----------------- Déclenchement sur Seuil -----------------
  if (Seuil) {
     if (Front_Montant) {
         //------- Attendre U en entrée minimale ------
         U_Actuelle = analogRead(A3) >> 2;
         while ((analogRead(A3) >> 2) > U_Actuelle)
               {U_Actuelle = analogRead(A3) >> 2;}
         while ((analogRead(A3) >> 2) <= U_Actuelle) {
               U_Actuelle = analogRead(A3) >> 2;} // Attendre la valeur minimale.
         //--------------------------------------------
         while ((analogRead(A3) >> 2) < U_Synchronisation) {}} // Attendre le Seuil.        
     else { //------- Attendre U en entrée minimale ------
         while ((analogRead(A3) >> 2) < U_Actuelle)
               {U_Actuelle = analogRead(A3) >> 2;}
         while ((analogRead(A3) >> 2) >= U_Actuelle) {
               U_Actuelle = analogRead(A3) >> 2;} // Attendre la valeur maximale.
         //--------------------------------------------
         while ((analogRead(A3) >> 2) > U_Synchronisation) {}}} // Attendre le Seuil.
  //----------------- Déclenchement sur Front -----------------
  else {if (Front_Montant) while ((analogRead(A3) >> 2) < 128); // Ne rien faire.
        else while ((analogRead(A3) >> 2) > 128);}} // Ne rien faire.

//=============================================================================

void Saisie_des_echantillons() {
  // Gain représente le nombre de décalages à droite donc pour
  // Gain = 2,1,0 le signale sera "multiplié" par 0 2 et 4.
  boolean FINI = false; boolean SAV_Grille = Grille; Mode_CAN = true; Num_Fenetre = 1;
  Automatique = true; Grille = false; Libre = true; Gain = 2; // 5V par défaut.
  while (!FINI) {
     if (!Libre) Gere_le_Declenchement();
     Effectue_une_numerisation();
     Tester_le_BP();
     switch (Touche) {
        case HAUT : {if (Clic_long) {Gain--; if (Gain == 255) Gain = 2;}
                     else Augmente_la_BT(); break;}
        case BAS : {if (Clic_long)  Grille = !Grille; else Diminue_la_BT(); break;}
        case GAUCHE : {if (Clic_long) Gerer_la_synchronisation();
                       else {if (Automatique) Libre = !Libre;} break;}
        case DROITE  : if (Clic_long) FINI = true;
                      else {Automatique = !Automatique;
                            if (!Automatique) digitalWrite(Trp_R,HIGH);}}
     u8g.firstPage(); do {
        u8g.setPrintPos(7,10); Affiche_le_Gain(); Affiche_la_duree(); u8g.print(' ');
        if (Automatique) u8g.print('A'); else u8g.print('M'); u8g.print(' ');
        if (Libre) u8g.print('L');
        else {if (Seuil) u8g.print('N'); else u8g.print('B');
              u8g.print(' '); if (Front_Montant) u8g.print('M'); else u8g.print('D');}
        if (Grille) Trace_la_grille(); Trace_la_courbe();}
     while(u8g.nextPage());
     if (!Automatique) {Attendre_un_BP();
                        if (Touche == DROITE) {Automatique = true;  digitalWrite(Trp_R,LOW);}}}
   Mode_CAN = false; Grille = SAV_Grille; Mode_Fenetre = false;}

//=============================================================================

void Aff_TRACE_H_B_Sortie() {
     Aff_TEXTE_EEPROM(216,7); u8g.print('?'); u8g.setPrintPos(20,31); // " TRACE ?"
     Aff_HAUT_deux_points_espace_OUI_Point(); // "HAUT : OUI."
     u8g.setPrintPos(20,43); Aff_BAS_deux_points_espace();
     Aff_NON(); Point(); // "NON."
     u8g.setPrintPos(20,55); DROITE_deux_points_espace_Sortie_point();} // "DROITE : Sortie."

void Effacer_TRACE_en_EEPROM() {
  boolean FINI = false;
  Eteindre_LED_Triple(); digitalWrite(Trp_R,HIGH);
  if (Lire_Octet_EEPROM(547) == 0) {BIP();
         u8g.firstPage(); do {
            u8g.setPrintPos(10,37); Aff_Pas_de_TRACE();} // "Pas de TRACE !"
         while(u8g.nextPage()); Attendre_un_BP();}
  else {while (!FINI) {
           Tester_le_BP();
           switch (Touche) {
              case HAUT : {{Ecrire_Octet_EEPROM(547,0); // Indique l'effacement de l'EEPROM.
                            for (int Echantillon = 548; Echantillon < 1023; Echantillon++)
                                 Ecrire_Octet_EEPROM(Echantillon,0); FINI = true;} break;}
              case BAS : {FINI = true; break;}
              case GAUCHE : {BIP(); break;}
              case DROITE  : FINI = true;}
              u8g.firstPage(); do {
                 u8g.setPrintPos(20,13); Aff_TEXTE_EEPROM(224,7); // "Effacer"
                 Aff_TRACE_H_B_Sortie();}
              while(u8g.nextPage());} Eteindre_LED_Triple(); digitalWrite(Trp_V,HIGH);}}

void Sauvegarde_Longue_TRACE() {
  boolean FINI = false; boolean Fini = false; boolean Accepte = true;
  if (Lire_Octet_EEPROM(547) == 1) {// Demander confirmation si 4 traces en EEPROM.
     BIP(); Accepte = false;
     while (!Fini) {
       Tester_le_BP();
       switch (Touche) {
          case HAUT : {Fini = true; Accepte = true; break;}
          case BAS : {FINI = true; Fini = true; break;}
          case GAUCHE : {BIP(); break;}
          case DROITE  : FINI = true; Fini = true;}
     u8g.firstPage(); do {
        u8g.setPrintPos(6,13); Aff_TEXTE_EEPROM(288,14); // "ATTENTION : 4 "
        Aff_TEXTE_EEPROM(217,5); Aff_TEXTE_EEPROM(323,2); // "TRACES."
        u8g.setPrintPos(12,25); Aff_TEXTE_EEPROM(302,17); // "On les remplace ?"
        u8g.setPrintPos(20,37); Aff_HAUT_deux_points_espace_OUI_Point();
        u8g.setPrintPos(20,49); Aff_BAS_deux_points_espace(); Aff_NON();
        u8g.setPrintPos(20,61); DROITE_deux_points_espace_Sortie_point();}
     while(u8g.nextPage());}}
  //------------ Ci-dessous sauvegarde la TRACE longue --------------
  if(Accepte) {u8g.firstPage(); do {
             u8g.setPrintPos(0,37); Aff_TEXTE_EEPROM(261,10); // "Sauvegarde"
             Aff_TEXTE_EEPROM(319,4); Aff_TEXTE_EEPROM(231,6); // " la Longue"
             Point();}
             while(u8g.nextPage());
             Ecrire_Octet_EEPROM(547, 2); // Signale longue trace en EEPROM.
             Ecrire_Octet_EEPROM(545, Gain);
             Ecrire_Octet_EEPROM(546, I_BdT_1);
             for (int Echantillon = 0; Echantillon < 476; Echantillon++)
                  Ecrire_Octet_EEPROM(Echantillon + 548, TRACE[Echantillon]);
             delay(1000);}} // Délai pour avoir le temps de lire le texte.

void Enregistrer_une_FENETRE() {
  int Decalage = 429 + (119 * Num_Fenetre); // 429 = 548 - 119.
  Calcule_le_Debut(); // Détermine le Début de la FENETRE.
  //--------------- Ci-dessous sauvegarde la FENETRE -----------------
  for (int Echantillon = 0; Echantillon < 119; Echantillon++)
      Ecrire_Octet_EEPROM(Echantillon + Decalage, TRACE[Echantillon + Debut]);}

void Sauvegarde_FENETRE() {
   u8g.firstPage(); do {
     u8g.setPrintPos(20,37); Aff_TEXTE_EEPROM(261,10); // "Sauvegarde"
     Aff_TEXTE_EEPROM(319,4); u8g.print(Num_Fenetre); // " la " et le n° de la fenêtre. 
     Point();} while(u8g.nextPage());
  Ecrire_Octet_EEPROM(547, 1); // Signale quatre traces individuelles en EEPROM.
  //--------------- Sauvegarde les attributs de la FENETRE --------------
  switch (Num_Fenetre) {
     case 1 : {Ecrire_Octet_EEPROM(539,Gain); Ecrire_Octet_EEPROM(540,I_BdT); break;}
     case 2 : {Ecrire_Octet_EEPROM(541,Gain); Ecrire_Octet_EEPROM(542,I_BdT); break;}
     case 3 : {Ecrire_Octet_EEPROM(543,Gain); Ecrire_Octet_EEPROM(544,I_BdT); break;}
     case 4 : {Ecrire_Octet_EEPROM(545,Gain); Ecrire_Octet_EEPROM(546,I_BdT);}}
  //---------------------------------------------------------------------
  Enregistrer_une_FENETRE(); delay(1000);} // Pour avoir le temps de lire le texte.

void Menu_Sauvegarde_TRACE() {
  boolean FINI = false; Eteindre_LED_Triple(); digitalWrite(Trp_R,HIGH);
  while (!FINI) {
     Tester_le_BP();
     switch (Touche) {
        case HAUT : {Sauvegarde_Longue_TRACE(); FINI = true; break;}
        case BAS : {Sauvegarde_FENETRE(); FINI = true; break;}
        case GAUCHE : {Num_Fenetre++; if (Num_Fenetre == 5) Num_Fenetre = 1; break;}
        case DROITE : FINI = true;}
     u8g.firstPage(); do {
        u8g.drawFrame(100,27, 13,14); u8g.drawHLine(80, 33, 20);
        u8g.drawHLine(92, 46, 14); u8g.drawVLine(106, 41, 6);
        u8g.setPrintPos(104,37); u8g.print(Num_Fenetre);
        u8g.setPrintPos(10,13); Aff_TEXTE_EEPROM(261,10); // "Sauvegarde"
        Aff_TEXTE_EEPROM(216,7); u8g.print('?'); // " TRACE ?"
        u8g.setPrintPos(0,25); Aff_TEXTE_EEPROM(13,7); // "HAUT : "
        Aff_TEXTE_EEPROM(273,9); // "Complète."
        u8g.setPrintPos(0,37); Aff_BAS_deux_points_espace();
        Aff_TEXTE_EEPROM(200,7); // "fenêtre"
        u8g.setPrintPos(0,49); Aff_TEXTE_EEPROM(41,9); // "GAUCHE : "
        Aff_TEXTE_EEPROM(282,6); // "Change");
        u8g.setPrintPos(0,61); Aff_TEXTE_EEPROM(62,9); // "DROITE : "
        Aff_TEXTE_EEPROM(156,7);} // "Sortie."
     while(u8g.nextPage());} Eteindre_LED_Triple(); }

void Recharge_un_enregistrement() {
  Num_Fenetre = 1; Fenetre = false;
   Mode_Fenetre = Lire_Octet_EEPROM(547) == 1;
  //------------------- Récupère les quatre paires d'attributs -----------------
  Gain_1 = Lire_Octet_EEPROM(539); Gain_2 = Lire_Octet_EEPROM(541);
  Gain_3 = Lire_Octet_EEPROM(543); Gain_4 = Lire_Octet_EEPROM(545);
  I_BdT_1 = Lire_Octet_EEPROM(540); I_BdT_2 = Lire_Octet_EEPROM(542);
  I_BdT_3 = Lire_Octet_EEPROM(544); I_BdT_4 = Lire_Octet_EEPROM(546);
  //----- Si traces individuelles imposer les attributs de la fenetre n°1. -----
  if (Lire_Octet_EEPROM(547) == 1) {Gain = Gain_1; I_BdT = I_BdT_1; Fenetre = true;}
  //--------- Si traces longue imposer les attributs de la position n°4. -------
  else {Gain = Gain_4; I_BdT = I_BdT_4; Fenetre = false;}
  //----------------------------------------------------------------------------
  for (int Echantillon = 548; Echantillon < 1024; Echantillon++)
        TRACE[int(Echantillon - 548)] = Lire_Octet_EEPROM(Echantillon);}

void Recharger_TRACE() {
  boolean FINI = false;
  digitalWrite(Trp_B,HIGH);
  if (Lire_Octet_EEPROM(547) == 0) {BIP();
         u8g.firstPage(); do {
            u8g.setPrintPos(10,37); Aff_Pas_de_TRACE();} // "Pas de TRACE !"
         while(u8g.nextPage()); Attendre_un_BP();}
  else {while (!FINI) {
           Tester_le_BP();
           switch (Touche) {
              case HAUT : {{Recharge_un_enregistrement(); FINI = true;} break;}
              case BAS : {FINI = true; break;}
              case GAUCHE : {BIP(); break;}
              case DROITE  : FINI = true;}
              u8g.firstPage(); do {
                 u8g.setPrintPos(20,13); Aff_TEXTE_EEPROM(251,9); // "Recharger"
                 Aff_TRACE_H_B_Sortie();}
              while(u8g.nextPage());}} digitalWrite(Trp_B,LOW);}

void Afficher_nature_TRACE() {
  u8g.print('(');
  switch (Lire_Octet_EEPROM(547)) {
        case 0 : {Aff_Pas_de_TRACE(); u8g.print(')'); break;} // "Pas de TRACE !)"
        case 2 : {Aff_TEXTE_EEPROM(231,7); Aff_TEXTE_EEPROM(217,5); // "Longue TRACE"
                  Aff_TEXTE_EEPROM(324,2); break;} // ".)"
        case 1 : {Aff_TEXTE_EEPROM(238,6); Aff_TEXTE_EEPROM(216,6);} // "Quatre TRACE"
  Aff_TEXTE_EEPROM(323,3);}} // "S.)"

void Menu_EEPROM() {
  boolean FINI = false; digitalWrite(Trp_V,HIGH);
  while (!FINI) {
     Tester_le_BP();
     switch (Touche) {
        case HAUT : {Effacer_TRACE_en_EEPROM(); FINI = true; break;}
        case BAS : {BIP();break;}
        case GAUCHE : {Recharger_TRACE(); FINI = true; break;}
        case DROITE  : if (Clic_long) FINI = true; else {Menu_Sauvegarde_TRACE(); FINI = true;}}
     u8g.firstPage(); do {
        u8g.setPrintPos(20,13);
        Aff_TEXTE_EEPROM(0,5); Aff_TEXTE_EEPROM(244,6); Point();// " MENU EEPROM."
        u8g.setPrintPos(12,25);
        Aff_TEXTE_EEPROM(13,7); Aff_TEXTE_EEPROM(224,7); Point(); // "HAUT : Effacer.");
        u8g.setPrintPos(0,37); Aff_TEXTE_EEPROM(41,9); // "GAUCHE : "
        Aff_TEXTE_EEPROM(251,10); // "Recharger."
        u8g.setPrintPos(0,49); Aff_TEXTE_EEPROM(62,9); // "DROITE : "
        Aff_TEXTE_EEPROM(261,12); // "Sauvegarder."
        u8g.setPrintPos(12,61); Afficher_nature_TRACE();}
     while(u8g.nextPage());} Eteindre_LED_Triple();}

//=============================================================================

void Affiche_duree_totale() {
  if (Total >= 1000) {u8g.print(Total / 1000); u8g.print('S');}
  else {u8g.print(Total); Aff_TEXTE_EEPROM(86,2);}} // "mS"

void Menu_duree_BdT() {
  boolean FINI = false;
  while (!FINI) {
    u8g.firstPage(); do {
    u8g.setPrintPos(23,13); Aff_TEXTE_EEPROM(171,13); // "BASE de TEMPS"
    u8g.setPrintPos(12,29); Aff_TEXTE_EEPROM(184,8); // "Durée : "
    Affiche_la_duree(); u8g.setPrintPos(12,44); Aff_TEXTE_EEPROM(192,8); // "Total : "
    Total = Grandeur * 40; Affiche_duree_totale(); u8g.setPrintPos(0, 58);
    Aff_TEXTE_EEPROM(200,10); // "Fenêtre : "
    Total = Grandeur * 10; Affiche_duree_totale();}
    while(u8g.nextPage());
  Attendre_un_BP();
  switch (Touche) {
    case HAUT : {Augmente_la_BT(); break;}
    case BAS  : {Diminue_la_BT(); break;}
    case GAUCHE : {BIP(); break;}
    case DROITE  : FINI = true;}}}

void Point() {u8g.print('.');}

void Aff_MENU_de_BASE() {
  digitalWrite(Trp_B,HIGH);
  u8g.firstPage(); do {
      u8g.setPrintPos(20,13); Aff_TEXTE_EEPROM(0,13); // "MENU de BASE."
      u8g.setPrintPos(0,25); Aff_HAUT_deux_points_espace(); // "HAUT : "
      Affiche_numeriser(); Point();
      u8g.setPrintPos(0,37); Aff_TEXTE_EEPROM(20,21); // "BAS : Afficher TRACE."
      u8g.setPrintPos(0,49); Aff_TEXTE_EEPROM(41,21); // "GAUCHE : Menu EEPROM."
      u8g.setPrintPos(0,61); Aff_TEXTE_EEPROM(62,20);} // "DROITE : Sortie USB."
  while(u8g.nextPage()); Eteindre_LED_Triple();}

void Gerer_le_MENU_de_BASE() {
  switch (Touche) {
          case DROITE : {BIP(); break;}
          case GAUCHE : {Menu_EEPROM(); break;}
          case HAUT  : {Saisie_des_echantillons(); break;}
          case BAS   : Traite_affichage();}}

void Gere_stabilisation_et_duree() {
  byte COMPTEUR; unsigned long Horloge_actuelle;
  Bruiteur(10); // Bip court.
  Horloge_actuelle = millis(); Clic_long = false; COMPTEUR = 0;
  //----------------------- Attendre le relâcher ---------------------- 
  while (COMPTEUR < NB_tests_antirebonds) {
         COMPTEUR++; // Tant que l'un des BP est activé le compteur est maintenu à zéro :
         SAV_A6 = analogRead(BP_A6); SAV_A7 = analogRead(BP_A7);
         if (SAV_A6 < 243 || SAV_A6 > 707 || SAV_A7 < 242 || SAV_A7 > 708) COMPTEUR = 0;
  //-------------------------------------------------------------------
  if ((millis() - Horloge_actuelle) > 500) Clic_long = true;
  if (Clic_long) digitalWrite(Jaune,HIGH);}
  digitalWrite(Jaune,LOW);}

void Tester_le_BP() {
  boolean BP_actif = false; Touche = 0;
  SAV_A6 = analogRead(BP_A6); if (SAV_A6 < 243) {
                                  Touche = DROITE; BP_actif = true;}
                              if (SAV_A6 > 707) {
                                  Touche = HAUT; BP_actif = true;}
  SAV_A7 = analogRead(BP_A7); if (SAV_A7 < 242) {
                                  Touche = BAS; BP_actif = true;}
                              if (SAV_A7 > 708) {
                                  Touche = GAUCHE; BP_actif = true;}
  // ---------- Si le B.P. est cliqué alors il est traité  ---------- 
  if (BP_actif) Gere_stabilisation_et_duree();}


  
